#include <ZXMod.h>

DESC="My implementation of the classic so called Lyapunov formula, i.e "
     "mapping of dynamics for x -> rx(1-x), both r and x being Real numbers, "
     "the parameter 'r' receiving alternate values by the plane's x and y in "
     "a cyclical fashion. X starts from a fixed value of 0.5, 500 pre-iterations "
     "are computed at beginning to help dynamics to better stabilize on the attractor. "
     "Use Julia map mode. Cx and Cy are treated as integers, "
     "where Cx is a binary mask (a bit ON means use y, a bit OFF means use x) "
     "and Cy is the mask length minus one. Please see the included document inside "
     "the module drawer for more info and instructions.\n"
     "NOTE: This is the same Lyap module I once published on Aminet. "
     "Originally they were two, the only difference between the two "
     "being that the second had just x and y swapped in function declaration "
     "(to avoid having to turn your head by 90 degrees, since ZX has no rotation "
     "capabilities) - Now they are merged into one same module, by just swapping "
     "the Lyapunov masks.";

#define PREITERS 500

ULONG FUNC(px,py,_msk,_ml,t,it)
{
 register DOUBLE x,x0,r,l,l1,l2,l3;
 register ULONG msk,ml,c;

 msk=(ULONG)_msk;
 ml=(ULONG)_ml +1;

 x=.5; l=l1=l2=l3=1;

 #define FX r*x*(1-x)
 #define DX (r-2*r*x)

 c=0;

 while(c<PREITERS)
 {
  if(msk & (1<<(c%ml))) r=py; else r=px; c++;
   x=FX; if(fabs(x)>100) RGBRETURN(0,0,0);
 }

 r=DX;
 c=0;
 while(c<it)
 {
  x0=r;
  if(msk & (1<<(c%ml))) r=py; else r=px; c++;
  x=FX; if(fabs(x)>100) RGBRETURN(0,0,0);
  r=DX;
  l1*=(r-x0); l*=r;

  if(msk & (1<<(c%ml))) r=py; else r=px; c++;
  x=FX;  if(fabs(x)>100) RGBRETURN(0,0,0);
  r=DX;
  l2*=(r-x0); l*=r;

  if(msk & (1<<(c%ml))) r=py; else r=px; c++;
  x=FX;  if(fabs(x)>100) RGBRETURN(0,0,0);
  r=DX;
  l3*=(r-x0); l*=r;
 }


 l=fabs(l); l1=fabs(l1); l2=fabs(l2); l3=fabs(l3);
 if(l==0) l=1E-300;
 if(l1==0) l1=1E-300;
 if(l2==0) l2=1E-300;
 if(l3==0) l3=1E-300;

 l=fabs(log(l)/it)/t; if(l>1) l=1; l=1-l;
 l1=fabs(log((l1))/it)/t; if(l1>1) l1=1; l1=1-l1;
 l2=fabs(log((l2))/it)/t; if(l2>1) l2=1; l2=1-l2;
 l3=fabs(log((l3))/it)/t; if(l3>1) l3=1; l3=1-l3;

 RGBRETURN(
  (UBYTE)((l1*(1-l)+l)*l*255),
  (UBYTE)((l2*(1-l)+l)*l*255),
  (UBYTE)((l3*(1-l)+l)*l*255)
 );

 /*RGBRETURN( (UBYTE)(l1*255),(UBYTE)(l2*255),(UBYTE)(l3*255));*/

}
