#include <ZXMod.h>

DESC="Example of a greyscale Module for Z+C using an optimized log map with simulated "
     "escape time (for a faster computation and to properly handle the infinite attractor, "
     "which would otherwise saturate the accumulator quickly.)\n"
     " Elena 2017";

#define N 2

ULONG FUNC(x,y,a,b,t,iters)
{
 register DOUBLE e,r,r0,acc,tmp;
 register ULONG c;

 r=x*x+y*y; // Initial radius (squared, it doesn't matter)
 acc=0.0;

 for(c=0;c<iters;c++)
 {
  r0=r;

  tmp=x*x-y*y+a; y=2*x*y+b; x=tmp; // Z=Z+C, our main function

  r=x*x+y*y;
  if(r>1000) // if r is sufficiently big we can approximate (simulate)
             // the escape dynamics, and the base will approximate better and better the
             // one used by the formula (N=2 in this case)
  {
   e=N * log(r0);
   for(;c<iters;c++) {acc+=e; e*=N;}
   break;
  }
  else {acc+= log(r);}
 }

 tmp=(cos(log(log(acc*acc)+1)*t)+1); // The secod Log just serves to attenuate the spreading
                                     // of outer gradients and to amplify that of inner ones

 c=(LONG)(tmp*127.5+0.5);
 RGBRETURN(c,c,c);
}

