// -------------------------------------------------
//        ZoneXplorer VBCC module header (PPC)
//                Elena-N 2004-2017
// -------------------------------------------------
// !!!    WARNING: NEVER MODIFY THIS FILE        !!!
// !!! UNLESS U REALLY KNOW WHAT U ARE DOING     !!!
// !!!  For precaution, this file has been       !!!
// !!!        protected from deletion            !!!
// -------------------------------------------------

// Convenient types

typedef unsigned short UWORD;
typedef unsigned long  ULONG;
typedef short WORD;
typedef long LONG;
typedef unsigned char UBYTE;
typedef char BYTE;
typedef double DOUBLE;


// Not really used but referenced by some linker libs

int errno;

// Main module function MUST be named "ZXFunc" and have this EXACT format,
// where:
// - a,b = point 1 coordinates;  c,d = point 2 coordinates
// (In Julia Mode: point 1 = point on the plane, point 2 = C constant)
// (in Mandel Generalized Mode: point 1 = point 2 = point on the plane)
// (in Mandel Mode With Z0: point 1 = Z0 constant, point 2 = point on the plane) 
// - e = user parameter (usually "threshold")
// - f = integer user parameter (usually number of iterations)

#define FUNC(a,b,c,d,e,f) ZXFunc(DOUBLE a, \
                                 DOUBLE b, \
                                 DOUBLE c, \
                                 DOUBLE d, \
                                 DOUBLE e, \
                                 ULONG  f) \

// The Module function shall return the computed pixel color value as an
// xRGB longword (x not used). The following convenient macro should be used:

#define RGBRETURN(r,g,b) return(((r)<<16)|((g)<<8)|(b))

// An additional string or whole text block containing module description,
// information, notices, instructions, curiosity or whatever else can be
// added to a module, if needed

#define DESC const char Description[]

// Commonly used transcendent constants

#define PID2  1.57079632679489661923
#define PI    3.14159265358979323846
#define PI2   6.28318530717958647692
#define E     2.718281828459
#define THETA 1.4446678610098          // Theta = e^(1/e) = exp(exp(-1)) = e^^(2+slog(-1))


// The various math functions from the linker math library
// (I avoided to list some I never use though)

extern DOUBLE cos(DOUBLE);
extern DOUBLE sin(DOUBLE);
extern DOUBLE log(DOUBLE);
extern DOUBLE sqrt(DOUBLE);
extern DOUBLE exp(DOUBLE);
extern DOUBLE pow(DOUBLE,DOUBLE);
extern DOUBLE atan(DOUBLE);
extern DOUBLE atan2(DOUBLE,DOUBLE);
extern DOUBLE acos(DOUBLE);
extern DOUBLE asin(DOUBLE);
extern DOUBLE tan(DOUBLE);
extern DOUBLE ceil(DOUBLE);
extern DOUBLE floor(DOUBLE);
extern DOUBLE fmod(DOUBLE,DOUBLE);
extern DOUBLE ldexp(DOUBLE,int);
extern DOUBLE frexp(DOUBLE,int *);

// These are supported by direct PPC instructions

DOUBLE        fabs(__reg("f1") DOUBLE) = "\tfabs\t1,1";
DOUBLE        fres(__reg("f1") DOUBLE) = "\tfres\t1,1";
DOUBLE        frsqrte(__reg("f1") DOUBLE) = "\tfrsqrte\t1,1";



// Some convenient macros to work with Complex Numbers (Z=x+iy; C=a+bi)
// Be aware: variable r (used as spare) must be declared

#define INV(x,y) {r=1/(x*x+y*y); x*=r; y=-y*r;}     // Z=1/Z
#define CZ {r=a*x-b*y; y=a*y+b*x; x=r;}             // Z=CZ  
#define Z2 {r=x*x-y*y; y=2*x*y; x=r;}               // Z=Z
#define Z3 {r=x*x*x-3*x*y*y; y=3*x*x*y-y*y*y; x=r;} // Z=Z
#define EZ {r=exp(x); x=r*cos(y); y=r*sin(y);}      // Z=exp(Z)
// Z=Z/C 
#define CDIV(x,y,a,b) {DOUBLE _r,_x1; _r=1.0/(a*a+b*b); _x1=_r*(a*x+b*y); y=_r*(a*y-b*x); x=_x1;}
// Z=Z rotated by m radians
#define ROT(x,y,m) {DOUBLE _cosm = cos(m), _sinm=sin(m);  r=_cosm*x-_sinm*y; y=_cosm*y+_sinm*x; x=r;}

// More useful math marcos

#define ARG(x,y) (((y<0)?(PI+PID2):(PID2))-atan((x)/(y)))
#define SGN(x) ((x<0)?(-1):(1))
#define LIM(x,m) {if(x>=m) x=m; if(x<-m) x=-m;}


// Author's private inlines

#pragma only-inline on

static DOUBLE red(DOUBLE x)
{x=fabs(x*6-3)-1; if(x>1) x=1; if(x<0) x=0; return(x);}

static DOUBLE grn(DOUBLE x)
{x=2-fabs(x*6-2); if(x>1) x=1; if(x<0) x=0; return(x);}

static DOUBLE blu(DOUBLE x)
{x=2-fabs(x*6-4); if(x>1) x=1; if(x<0) x=0; return(x);}

static DOUBLE max(DOUBLE a, DOUBLE b)
{return(a>b ? a : b);}

static DOUBLE min(DOUBLE a, DOUBLE b)
{return(a<b ? a : b);}

#pragma only-inline off
